; File        : a2600.S
; Version     : 0.0
; Description : crt0.s style setup for cc65 and Atari 2600 VCS
; Author      : Adam Wozniak <adam@cuddlepuddle.org>
; Date        : Tue Mar 26 18:04:32 PST 2002
;
; This file is part of cc65-a2600lib.
; 
; cc65-a2600lib is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; cc65-a2600lib is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with Foobar; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

.autoimport on

.import _main
.import _irq
.import _nmi

; Import linker generated symbols
.import __CALLSTACK_START__, __CALLSTACK_SIZE__;
.import __PARMSTACK_START__, __PARMSTACK_SIZE__;
.import __BSS_RUN__, __BSS_SIZE__;
.import __DATA_LOAD__, __DATA_RUN__, __DATA_SIZE__;

.export _exit
.export start

.exportzp sp, sreg, regsave
.exportzp ptr1, ptr2, ptr3, ptr4
.exportzp tmp1, tmp2, tmp3, tmp4
.exportzp regbank, zpspace

TOPMEM = $df

.zeropage
 
zpstart  = *
sp:            .res  2  ; Stack pointer
sreg:          .res  2  ; Secondary register/high 16 bit for longs
regsave:       .res  2  ; slot to save/restore (E)AX into
ptr1:          .res  2
ptr2:          .res  2
ptr3:          .res  2
ptr4:          .res  2
tmp1:          .res  1
tmp2:          .res  1
tmp3:          .res  1
tmp4:          .res  1
regbank:       .res  6  ; 6 byte register bank
 
zpspace  = * - zpstart     ; Zero page space allocated

.segment "CODE"

.proc start

   ; disable interrupts
   sei

   ; clear decimal bit
   cld

   ; Set the 6502 call stack as defined in the linker config
   ldx #<(__CALLSTACK_START__ + __CALLSTACK_SIZE__ - 1)
   txs

   ; Set the parameter stack as defined in the linker config
   lda #<(__PARMSTACK_START__ + __PARMSTACK_SIZE__)
   sta sp
   lda #>(__PARMSTACK_START__ + __PARMSTACK_SIZE__)
   sta sp+1

   ; zero out the bss RAM
zbss:
   ldx #<(__BSS_SIZE__)     ; set up X
   beq end_zbss             ; skip all this if zero
   lda #<(__BSS_RUN__)      ; set up ptr1
   sta ptr1
   lda #>(__BSS_RUN__)
   sta ptr1+1
   tay                      ; initialize index pointer
   lda #$00                 ; set up zero value
zbss1:
   sta (ptr1),Y             ; clear memory location
   iny                      ; increment index
   dex                      ; decrement counter
   bne zbss1                ; not zero, continue
end_zbss:

   ; copy the DATA segment into RAM
cdsr:
   ldx #<(__DATA_SIZE__)   ; set up X
   beq end_cdsr            ; skip all this if zero
   lda #<(__DATA_RUN__)    ; set up ptr1
   sta ptr1
   lda #>(__DATA_RUN__)
   sta ptr1+1
   lda #<(__DATA_LOAD__)    ; set up ptr2
   sta ptr2
   lda #>(__DATA_LOAD__)
   sta ptr2+1
   tay                      ; initialize index pointer
cdsr1:
   lda (ptr2),Y             ; get value from load area
   sta (ptr1),Y             ; put value in run area
   iny                      ; increment index
   dex                      ; decrement counter
   bne cdsr1                 ; not zero, continue
end_cdsr:

   ; clear out the registers
   ldx #0
   ldy #0
   lda #0

   ; set up the initial stack
   ; NOTE this assumes void main(void) !!!
   ldy #0
   jsr _main

   jmp start
.endproc

.segment	"CODE"

.segment "CODE"
.proc irq
   jsr _irq
   rti
.endproc

.segment "CODE"
.proc nmi
   jsr _nmi
   rti
.endproc

.segment "CODE"

.proc _exit
   ; disable interrupts
   sei

   jmp start
.endproc

.segment "DATA"

zpsave: .res zpspace

.segment "BSS"

spsave: .res 1

.segment "VECTORS"
.addr nmi
.addr start
.addr irq
